import { DataConnector } from '@jupyterlab/statedb';
export default class MergeConnector extends DataConnector {
    constructor(completionItemsConnector, dataConnector) {
        super();
        this.responseType = 'ICompletionItemsReply';
        this.completionItemsConnector = completionItemsConnector;
        this.dataConnector = dataConnector;
    }
    async fetch(request) {
        console.log('calling fetch');
        try {
            const replyWithItems = await this.completionItemsConnector.fetch(request);
            const replyWithMatches = await this.dataConnector.fetch(request);
            return mergeReplies(replyWithItems, replyWithMatches);
        }
        catch (ex) {
            console.log('error', ex);
            return Promise.reject(ex.toString());
        }
    }
}
export function mergeReplies(replyWithItems, replyWithMatches) {
    if (replyWithItems && replyWithMatches) {
        const { start } = replyWithItems;
        const { end } = replyWithItems;
        const items = [];
        replyWithItems === null || replyWithItems === void 0 ? void 0 : replyWithItems.items.forEach(item => items.push(item));
        const replyWithMatchesMetaData = replyWithMatches.metadata
            ._jupyter_types_experimental;
        replyWithMatches.matches.forEach((label, index) => items.push({
            label,
            type: replyWithMatchesMetaData
                ? replyWithMatchesMetaData[index].type
                : ''
        }));
        return { start, items, end };
    }
    return undefined;
}
